/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.forms.buttons');

org.ametys.forms.buttons.FormsButton = function(id, configuration, pluginName) 
{
	org.ametys.forms.buttons.FormsButton.superclass.constructor.call(this, id, configuration, pluginName);
};

Ext.extend(org.ametys.forms.buttons.FormsButton, org.ametys.ribbon.control.button.ActionButton, {});

/**
 * @private
 * @property _lastParams The params received on the last refresh
 */
org.ametys.forms.buttons.FormsButton.prototype._lastParams;

/**
 * @private
 * @property {String[]} _currentContentsId The content id currently selected or null. 
 */
org.ametys.forms.buttons.FormsButton.prototype._currentContentsId;

org.ametys.forms.buttons.FormsButton.prototype.onMessage = function (messages)
{
	// SELECTION
	for (var i=messages.length - 1; i >= 0; i--)
	{
		if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
		{
			var targets = messages[i].getTargets();
			var allTargetsObject = org.ametys.messagebus.message.MessageTargetHelper.findAll(targets, this.checkTarget());
			var allTargets = this._convertToId(allTargetsObject);
			
			// No content selected
			if (allTargets.length == 0)
			{
				this.getElements().each(function(element) { element.setDisabled(true); });
				this.updateTooltip(this.getConfiguration()["default-description"]);
				
				this._currentContentsId = [];
			}
			else
			{
				if (!this._areTheSame(this._currentContentsId, allTargets))
				{
					this._currentContentsId = allTargets;
					this.getElements().each(function(element) { element.refreshing(); element.setDisabled(true); });
					this.updateTooltip(this.getConfiguration()["default-description"]);
					org.ametys.ribbon.RibbonManager.getInstance().refreshElement('control', this, {targets: allTargets});
				}
			}
			
			return;
		}
		else if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_METADATACHANGED)
		{
			var targets = messages[i].getTargets();
			var target = org.ametys.messagebus.message.MessageTargetHelper.findFirst(targets, this.checkTarget());
			if (target != null)
			{
				if (this._isInSelection (this._currentContentsId, target.getParameters().id))
				{
					// Refresh
					this.getElements().each(function(element) { element.refreshing(); element.setDisabled(true); });
					this.updateTooltip(this.getConfiguration()["default-description"]);
					
					org.ametys.ribbon.RibbonManager.getInstance().refreshElement('control', this, {targets: this._currentContentsId});
				}
			}
		}
			
	}
}

org.ametys.forms.buttons.FormsButton.prototype._areTheSame = function (tab1, tab2)
{
	if (tab1 == tab2)
	{
		return true;
	}

	if (tab1 == null || tab2 == null || tab1.length != tab2.length)
	{
		return false;
	}	
	
	for (var i=0; i < tab1.length; i++)
	{
		if (tab2.indexOf(tab1[i]) == -1)
		{
			return false;
		}
	}
	
	return true;
}

org.ametys.forms.buttons.FormsButton.prototype._isInSelection = function (tab, elmt)
{
	if (tab == null || tab.length == 0)
	{
		return false;
	}	
	
	for (var i=0; i < tab.length; i++)
	{
		if (tab[i] == elmt)
		{
			return true;
		}
	}
	
	return false;
	
}

org.ametys.forms.buttons.FormsButton.prototype._convertToId = function (targets)
{
	var targetsIds = [];
	for (var i = 0; i < targets.length; i++)
	{
		targetsIds.push(targets[i].getParameters()['id']);
	}
	return targetsIds;
}
org.ametys.forms.buttons.FormsButton.prototype.checkTarget = function ()
{
	return function (target) { return target.getType() == 'content'; };
}

org.ametys.forms.buttons.FormsButton.prototype.configurationError = function(response)
{
	this.getElements().each(function(element) { element.stopRefreshing(); element.setDisabled(true); });
	this.updateTooltip(this.getConfiguration()["default-description"]);
}

org.ametys.forms.buttons.FormsButton.prototype.configurationChanged = function(params)
{
	this.getElements().each(function(element) { element.stopRefreshing(); });
	
	var tooltip = this._getTooltip("", params);
	
	var hasOneContent = tooltip.length > 0; 
	if (!hasOneContent)
	{
		tooltip += this.getConfiguration()['error-description'];
	}
	
	var toggleState = false;
	this.getElements().each(function(element) { element.allowDepress = false; element.toggle(toggleState); });
	
    var disabledState = this._isDisabled(params);
    this.getElements().each(function(element) { element.setDisabled(disabledState); });
    
	this._getIcon(params);
	
	this.getElements().each(function(element) { element.updateIcon(); });
	this.updateTooltip(tooltip, this.getElements().get(0).iconLarge, null);
	
	this._lastParams = params;
}

org.ametys.forms.buttons.FormsButton.prototype._isToggled = function(params)
{
	return false;
}

org.ametys.forms.buttons.FormsButton.prototype._isDisabled = function(params)
{
    var formCount = parseInt(params['form-count']);
	
    return formCount < 1;
}

org.ametys.forms.buttons.FormsButton.prototype._getIcon = function(params)
{
	var formCount = parseInt(params['form-count']);
	
	var iconSmall;
	var iconLarge;
	
//	if (formCount > 0)
//	{
	iconSmall = context.contextPath + this.getConfiguration()['icon-small'];
	iconLarge = context.contextPath + this.getConfiguration()['icon-medium'];
//	}
//	else
//	{
//        iconSmall = context.contextPath + this.getConfiguration()['no-forms-icon-small'];
//        iconLarge = context.contextPath + this.getConfiguration()['no-forms-icon-medium'];
//	}

    this.getElements().each(
        function (element) 
        {
            element.iconSmall = iconSmall;
            element.iconLarge = iconLarge;
        }
    );
}

org.ametys.forms.buttons.FormsButton.prototype._getTooltip = function(tooltip, params)
{
    var formCount = parseInt(params['form-count']);
    
	var tooltip;
	
    if (formCount > 0)
    {
        tooltip = this.getConfiguration()['tooltip'];
    }
    else
    {
        tooltip = this.getConfiguration()['no-forms-tooltip'];
    }
	
	return tooltip;
}


